#!/usr/bin/php -q
<?php
/* do NOT run this script through a web browser */
if (!isset($_SERVER["argv"][0])) {
	die("<br><strong>This script is only meant to run at the command line.</strong>");
}

$no_http_headers = true;

#################################################
#
# Well, usage
#
function display_help() {
	echo "Add Device SOAP Script 1.0, Copyright 2007 - The Cacti Group\n\n";
	echo "A simple command line SOAP utility to add a device in Cacti\n\n";
	echo "usage: add_device_cl.php --username=[username] --password=[password]\n";
	echo "--urn=[urn] --do=[List Options]\n";
	echo "Add host options\n";
	echo "    --description=[description] --fqdn=[fqdn] --template-id=[ID] [--notes=\"[]\"] [--disable]\n";
	echo "    [--avail=[ping]] --ping_method=[icmp] --ping_port=[N/A, 1-65534] --ping_retries=[2]\n";
	echo "    [--snmp_ver=[1|2|3]] [--community=] [--port=161] [--timeout=500]\n";
	echo "    [--snmpusername= --snmppassword=] [--authproto=] [--privpass= --privproto=] [--context=]\n";
	echo "    [--quiet]\n\n";
	echo "Required:\n";
	echo "    --description  the name that will be displayed by Cacti in the graphs\n";
	echo "    --fqdn           self explanatory (can also be a FQDN)\n";
	echo "Optional:\n";
	echo "    --template-id  0, is a number (read below to get a list of templates)\n";
	echo "    --notes        '', General information about this host.  Must be enclosed using double quotes.\n";
	echo "    --disable      0, 1 to add this host but to disable checks and 0 to enable it\n";
	echo "    --avail        pingsnmp, [ping][none, snmp, pingsnmp]\n";
	echo "    --ping_method  tcp, icmp|tcp|udp\n";
	echo "    --ping_port    '', 1-65534\n";
	echo "    --ping_retries 2, the number of time to attempt to communicate with a host\n";
	echo "    --snmp_ver      1, 1|2|3, snmp version\n";
	echo "    --community    '', snmp community string for snmpv1 and snmpv2.  Leave blank for no community\n";
	echo "    --port         161\n";
	echo "    --timeout      500\n";
	echo "    --snmpusername     '', snmp username for snmpv3\n";
	echo "    --snmppassword     '', snmp password for snmpv3\n";
	echo "    --authproto    '', snmp authentication protocol for snmpv3\n";
	echo "    --privpass     '', snmp privacy passphrase for snmpv3\n";
	echo "    --privproto    '', snmp privacy protocol for snmpv3\n";
	echo "    --context      '', snmp context for snmpv3\n\n";
	echo "List Options:\n";
	echo "    --add-host [Add host options]\n";
	echo "    --update-fqdn --fqdn [fqdn] --description [description]\n";
	echo "    --update-description --description [description] --fqdn [fqdn]\n";
	echo "    --list-host-templates '',returns the valid host templates\n";
	echo "    --list-communities '', returns the known community strings\n\n";
}

#####################################################################
#
# Main
#
/* process calling arguments */
$parms = $_SERVER["argv"];
array_shift($parms);

if (!sizeof($parms)) {
	display_help();
	exit(1);
}else{
	/* setup defaults */
	$username = "";
	$password = "";
	$switch = "";

	$description 	= "";
	$fqdn 			= "";
	$template_id 	= 0;
	$community 		= "";
	$snmp_ver 		= 1;
	$disable 		= 0;
	
	$notes         	= "";
	
	$snmp_username 			= "";
	$snmp_password = "";
	$snmp_auth_protocol		= "";
	$snmp_priv_passphrase	= "";
	$snmp_priv_protocol		= "";
	$snmp_context			= "";
	$snmp_port				= 161;
	$snmp_timeout			= 5000;
	$max_oids 				= 60;
	
	$avail        = 1;
	$ping_method  = 3;
	$ping_port    = 23;
	$ping_timeout = 5000;
	$ping_retries = 3;

	foreach($parms as $parameter) {
		@list($arg, $value) = @explode("=", $parameter);
		
		switch($arg)
		{
			case "--username":
				$username = trim($value);
				break;
			case "--password":
				$password = trim($value);
				break;
			case "--urn":
				$urn = trim($value);
				break;
			case "--template-id":
				$template_id = trim($value);
				break;
			case "--description":
				$description = trim($value);
				break;
			case "--fqdn":
				$fqdn = trim($value);
				break;
			case "--community":
				$community = trim($value);
				break;
			case "--snmp_ver":
				$snmp_ver = trim($value);
				break;
			case "--disable":
				$disable = 1;
				break;
			case "--notes":
				$notes = trim($value);
				break;
			case "--snmpusername":
				$snmp_username = trim($value);
				break;
			case "--snmppassword":
				$snmp_password = trim($value);
				break;
			case "--authproto":
				$snmp_auth_protocol = trim($value);
				break;
			case "--privpass":
				$snmp_priv_passphrase = trim($value);
				break;
			case "--port":
				$snmp_port     = trim($value);
				break;
			case "--timeout":
				$snmp_timeout  = trim($value);
				break;
			case "--avail":
				switch($value) {
					case "none":
						$avail = 0;
						break;
					case "ping":
						$avail = 3;
						break;
					case "snmp":
						$avail = 2;
						break;
					case "pingsnmp":
						$avail = 1;
						break;
					default:
						display_help();
						echo "ERROR: Invalid Availability Parameter: ($value)\n\n";
						exit(1);
				}
				break;
    		case "--ping_method":
    			switch(strtolower($value)) {
        			case "icmp":
        				$ping_method = 1;
        				break;
        			case "tcp":
        				$ping_method = 3;
        				break;
        			case "udp":
        				$ping_method = 2;
        				break;
        			default:
        				display_help();
        				echo "ERROR: Invalid Ping Method: ($value)\n\n";
        				exit(1);
					}
					break;
			case "--do":
				$switch = $value;
				break;
    		case "--version":
    		case "-V":
    		case "-H":
    		case "--help":
    			display_help();
    			exit(0);
			default:
				display_help();
				exit(0);
		}
	}

	if ($username == "" || $password == ""){
		display_help();
		exit(1);
	}
	if ($urn == ""){
		display_help();
		exit(1);
	}
	$url_Location = 'http://' . $urn . '/soap_service.php';
	$urn_Location = 'urn:' . $urn . '';

	$opts = array('location' => $url_Location, 'uri' => 'pc_SOAP_return_soap');
	$client = new SOAPClient(null, $opts);

	$auth = new SOAPAuth ($username, $password);
	$header = new SOAPHeader($urn_Location, 'authenticate_user', $auth);

try {
	switch($switch)
	{
		case "--list-communities":
			$returnResult = $client->__soapCall('return_list_snmp_community_get_Communities', array(), array(), array($header));
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){display_help(); printThrowException($returnResult["Errors"]);exit(1);}
			displayCommunities($returnResult["Result"]);
			break;
		case "--list-host-templates":
			$returnResult = $client->__soapCall('return_list_IDName_get_Host_Templates', array(), array(), array($header));
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){display_help(); printThrowException($returnResult["Errors"]);exit(1);}
			displayHostTemplates($returnResult["Result"]);
			break;
		case "--update-fqdn":
			$returnResult = $client->__soapCall('return_ID_update_FQDN', array('fqdn' => $fqdn, 'description' => $description), array(), array($header));
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){display_help(); printThrowException($returnResult["Errors"]);exit(1);}
			echo "Device fqdn updated - device-id: (" . $returnResult["Result"] . ")";
			break;
		case "--update-description":
			$returnResult = $client->__soapCall('return_ID_update_Description', array('description' => $description, 'fqdn' => $fqdn), array(), array($header));
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){display_help(); printThrowException($returnResult["Errors"]);exit(1);}
			echo "Device description updated - device-id: (" . $returnResult["Result"] . ")";
			break;
		case "--add-host":
			$returnResult = $client->__soapCall('return_ID_add_Device', array(
			'template_id' => $template_id, 'description' => $description, 'fqdn' => $fqdn, 'community' => $community, 'snmp_ver' => $snmp_ver,
			'snmp_username' => $snmp_username, 'snmp_password' => $snmp_password, 'snmp_port' => $snmp_port, 'snmp_timeout' => $snmp_timeout,
			'disable' => $disable, 'avail' => $avail, 'ping_method' => $ping_method, 'ping_port' => $ping_port, 'ping_timeout' => $ping_timeout,
			'ping_retries' => $ping_retries, 'notes' => $notes, 'snmp_auth_protocol' => $snmp_auth_protocol, 'snmp_priv_passphrase' => $snmp_priv_passphrase,
			'snmp_priv_protocol' => $snmp_priv_protocol, 'snmp_context' => $snmp_context, 'max_oids' => $max_oids), array(), array($header));

			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){display_help(); printThrowException($returnResult["Errors"]);exit(1);}
			echo "Device Added - device-id: (" . $returnResult["Result"] . ")";
			break;
		default:
			display_help();
			break;
	}
	exit(0);
	
} catch (Exception $e) {
	echo ("Message = " . $e->__toString() . "\n");	
	exit(1);
}
	
}

function displayHostTemplates($host_templates)
{
	echo "Valid host templates:\n";
	foreach ($host_templates as $elemnt) {
		echo $elemnt["id"] . "\t" . $elemnt["name"] ."\n";
	}
}

function displayCommunities($communities)
{
	echo "Known communities are:\n";
	foreach ($communities as $elemnt) {
		echo $elemnt["id"] . "\t" . $elemnt["snmp_community"] ."\n";
	}
}

class SOAPAuth{
	public $username;
	public $password;

	public function __construct($username, $password){
		$this->username = $username;
		$this->password = $password;
	}
}

function printThrowException($errors){
	foreach($errors as $throwException){
		printf ("Error: " . $throwException["description"] . " - " . $throwException["function"] . "/" . $throwException["variable"] . "\n");
	}
}
?>
