#!/usr/bin/php -q
<?php
/* do NOT run this script through a web browser */
if (!isset($_SERVER["argv"][0])) {
	die("<br><strong>This script is only meant to run at the command line.</strong>");
}

$no_http_headers = true;

#################################################
#
# Well, usage
#
function display_help() {
	echo "Add Graphs SOAP Script 1.0, Copyright 2007 - The Cacti Group\n\n";
	echo "A simple command line utility to add graphs in Cacti\n\n";
	echo "usage: add_graphs.php --username=[username] --password=[password]\n";
	echo "--urn=[urn] --do=[List Options]\n";
	echo "--graph-template-id=[ID] --host-id=[ID] [--graph-title=title] [graph options]\n\n";
	echo "List Options:\n";
	echo "    --add-graph-cg [CG options]\n";
	echo "    --add-graph-ds [DS options]\n";
	echo "    --list-hosts\n";
	echo "    --list-graph-templates\n";
	echo "    --show-host-index --host-Name=[host-name]\n";
	echo "    --list-input-fields --graph-template-id=[ID]\n";
	echo "    --list-snmp-queries\n";
	echo "    --list-query-types  --snmp-query-id=[ID]\n";
	echo "    --list-snmp-fields  --host-id=[ID]\n";
	echo "    --list-snmp-values  --host-id=[ID] --snmp-field=[Field]\n\n";
	echo "DS options\n";
	echo "'ds' graphs are for data-source based graphs (interface stats etc.)\n";
	echo "For --add-graph-ds or --update-graph-ds graphs:\n";
	echo "    --snmp-query-id=[ID] --snmp-query-type-id=[ID] --snmp-field=[SNMP Field] --snmp-value=[SNMP Value]\n\n";
	echo "    [--graph-title=] Defaults to what ever is in the graph template/data-source template.\n\n";
	echo "CG options:\n";
	echo "'cg' graphs are for things like CPU temp/fan speed, while \n";
	echo "For --add-graph-cg or --update-graph-cg graphs:\n";
	echo "    [--input-fields=\"[data-template-id:]field-name=value ...\"] [--force]\n\n";
	echo "    --input-fields  If your data template allows for custom input data, you may specify that\n";
	echo "                    here.  The data template id is optional and applies where two input fields\n";
	echo "                    have the same name.\n";
	echo "    --force         If you set this flag, then new cg graphs will be created, even though they\n";
	echo "                    may already exist\n\n";
}

#####################################################################
#
# Main
#
/* process calling arguments */
$parms = $_SERVER["argv"];
array_shift($parms);

if (!sizeof($parms)){
	display_help();
	exit(1);
}else{
	$username 		= "";
	$password 		= "";
	$switch 		= "";

	$graphTitle 		= "";
	$hostId     		= 0;
	$template_id 		= 0;
	$snmpQueryId 		= 0;
	$snmpQueryTypeID 	= 0;
	$snmpFieldName 		= "";
	$snmpValueName 		= "";

	foreach($parms as $parameter) {
		@list($arg, $value) = @explode("=", $parameter, 2);
		switch($arg)
		{
			case "--username":
				$username = $value;
				break;
			case "--password":
				$password = $value;
				break;
			case "--urn":
				$urn = $value;
				break;
			case "--graph-title":
				$graphTitle = $value;
				break;
			case "--graph-template-id":
				$template_id = $value;
				break;
			case "--host-id":
				$hostId = $value;
				break;
			case "--snmp-query-id":
				$snmpQueryId = $value;
				break;
			case "--snmp-query-type-id":
				$snmpQueryTypeID = $value;
				break;
			case "--snmp-field":
				$snmpFieldName = $value;
				break;
			case "--snmp-value":
				$snmpValueName = $value;
				break;
			case "--do":
				$switch = $value;
				break;
			case "--host-Name":
				$hostName = $value;
				break;
			default:
				display_help();
				exit(0);
		}
	}

	if ($username == "" || $password == ""){
		display_help();
		exit(1);
	}
	if ($urn == ""){
		display_help();
		exit(1);
	}
	$url_Location = 'http://' . $urn . '/soap_service.php';
	$urn_Location = 'urn:' . $urn . '';

	$opts = array('location' => $url_Location,
	'uri' => 'pc_SOAP_return_soap');
	$client = new SOAPClient(null, $opts);

	$auth = new SOAPAuth ($username, $password);
	$header = new SOAPHeader($urn_Location, 'authenticate_user', $auth);

try{
	switch($switch)
	{
		case "--list-hosts":
			$returnResult = $client->__soapCall('return_list_IDHostname_get_Hosts', array(), array(), array($header));
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){printThrowException($returnResult["Errors"]);exit(1);}
			displayHosts($returnResult["Result"]);
			exit(0);
		case "--list-graph-templates":
			$returnResult = $client->__soapCall('return_list_IDName_get_Graph_Templates', array(), array(), array($header));
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){printThrowException($returnResult["Errors"]);exit(1);}
			displayGraphTemplates($returnResult["Result"]);
			exit(0);
		case "--show-host-index":
			$returnResult = $client->__soapCall('return_ID_get_Host', array('hostName' => $hostName), array(), array($header));
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){printThrowException($returnResult["Errors"]);exit(1);}
			$hostId = $returnResult["Result"];
			echo "Host (id, hostName):\n";
			echo "\t" . $hostId . "\t" . $hostName . "\n";
			exit(0);
		case "--list-snmp-queries":
			$returnResult = $client->__soapCall('return_list_IDName_get_SNMP_Queries', array(), array(), array($header));
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){printThrowException($returnResult["Errors"]);exit(1);}
			displaySNMPQueries($returnResult["Result"]);
			exit(0);
			break;
		case "--list-query-types":
			$returnResult = $client->__soapCall('return_list_IDName_get_SNMP_Query_Types', array('snmpQueryId' => $snmpQueryId), array(), array($header));
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){printThrowException($returnResult["Errors"]);exit(1);}
			displayQueryTypes($returnResult["Result"]);
			exit(0);
			break;
		case "--list-snmp-values":
			$returnResult = $client->__soapCall('return_list_Value_get_SNMP_Values', array('hostId' => $hostId, 'snmpFieldName' => $snmpFieldName), array(), array($header));
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){printThrowException($returnResult["Errors"]);exit(1);}
			displaySNMPValues($returnResult["Result"]);
			exit(0);
			break;
		case "--list-snmp-fields":
			$returnResult = $client->__soapCall('return_list_Field_get_SNMP_Fields', array('hostId' => $hostId), array(), array($header));
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){printThrowException($returnResult["Errors"]);exit(1);}
			displaySNMPFields($returnResult["Result"]);
			exit(0);
			break;
		case "--add-graph-ds":
			$returnResult = $client->__soapCall('return_ID_add_Graph_DS',
			array(	'graphTitle' => $graphTitle,
			'template_id'=> $template_id,
			'hostId' => $hostId,
			'snmpQueryId' => $snmpQueryId,
			'snmpQueryTypeID' => $snmpQueryTypeID,
			'snmpFieldName' => $snmpFieldName,
			'snmpValueName' => $snmpValueName), array(), array($header));
			 
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){display_help(); printThrowException($returnResult["Errors"]);exit(1);}
			
			foreach($returnResult["Result"] as $graphID){
				echo "Graph DS added - graph-id: (" . $graphID . ")\n";
			}
			exit(0);
			break;
		case "--add-graph-cg":
			$returnResult = $client->__soapCall('return_ID_add_Graph_CG',
			array(	'graphTitle' => $graphTitle,
			'template_id'=> $template_id,
			'hostId' => $hostId), array(), array($header));
			 
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){display_help(); printThrowException($returnResult["Errors"]);exit(1);}
			foreach($returnResult["Result"] as $graphID){
				echo "Graph CG added - graph-id: (" . $graphID . ")\n";
			}
			exit(0);
			break;
		case "--update-graph-ds":
			$returnResult = $client->__soapCall('return_ID_update_Graph_Title_DS',
			array(	'graphTitle' => $graphTitle,
			'template_id'=> $template_id,
			'hostId' => $hostId,
			'snmpQueryId' => $snmpQueryId,
			'snmpFieldName' => $snmpFieldName,
			'snmpValueName' => $snmpValueName), array(), array($header));
			 
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){display_help(); printThrowException($returnResult["Errors"]);exit(1);}
			echo "Graph title DS updated - graph-id: (" . $returnResult["Result"] . ")\n";
			exit(0);
			break;
		case "--update-graph-cg":
			$returnResult = $client->__soapCall('return_ID_update_Graph_Title_CG',
			array(	'graphTitle' => $graphTitle,
			'template_id'=> $template_id,
			'hostId' => $hostId), array(), array($header));
			 
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){display_help(); printThrowException($returnResult["Errors"]);exit(1);}
			echo "Graph title CD updated - graph-id: (" . $returnResult["Result"] . ")\n";
			exit(0);
			break;
		default:
			display_help();
			break;
	}
	exit(0);
	
} catch (Exception $e) {
	echo ("Message = " . $e->__toString() . "\n");
	exit(1);
}
}

function displaySNMPFields($fieldNames)
{
	echo "Known SNMP Fields:\n";
	foreach ($fieldNames as $elemnt) {
		echo $elemnt["field_name"] ."\n";
	}
}

function displaySNMPValues($fieldValues)
{
	echo "Known values SNMP Values:\n";
	foreach ($fieldValues as $elemnt) {
		echo $elemnt["field_value"] ."\n";
	}
}

function displayQueryTypes($types)
{
	echo "Known SNMP Query Types:(id, name)\n";
	foreach ($types as $elemnt) {
		echo $elemnt["id"] . "\t" . $elemnt["name"] ."\n";
	}
}

function displaySNMPQueries($queries)
{
	echo "Known SNMP Queries:(id, name)\n";
	foreach ($queries as $elemnt) {
		echo $elemnt["id"] . "\t" . $elemnt["name"] ."\n";
	}
}

function displayGraphTemplates($templates)
{
	echo "Known Graph Templates:(id, name)\n";
	foreach ($templates as $elemnt) {
		echo $elemnt["id"] . "\t" . $elemnt["name"] ."\n";
	}
}

function displayHosts($hosts)
{
	echo "Known Hosts:(id, hostname)\n";
	foreach ($hosts as $elemnt) {
		echo $elemnt["id"] . "\t" . $elemnt["hostname"] ."\n";
	}
}

class SOAPAuth{
	public $username;
	public $password;

	public function __construct($username, $password){
		$this->username = $username;
		$this->password = $password;
	}
}

function printThrowException($errors){
	foreach($errors as $throwException){
		printf ("Error: " . $throwException["description"] . " - " . $throwException["function"] . "/" . $throwException["variable"] . "\n");
	}
}

?>
