#!/usr/bin/php -q
<?php

/* do NOT run this script through a web browser */
if (!isset($_SERVER["argv"][0])) {
	die("<br><strong>This script is only meant to run at the command line.</strong>");
}

$no_http_headers = true;

#################################################
#
# Well, usage
#
function display_help() {
	echo "Add Tree SOAP Script 1.0, Copyright 2007 - The Cacti Group\n\n";
	echo "A simple command line utility to add objects to a tree in Cacti\n\n";
	echo "usage: add_tree.php --username=[username] --password=[password]\n";
	echo "--urn=[urn] --do=[List Options]\n";
	echo "Tree options:\n";
	echo "    --name=[Tree Name]\n";
	echo "    --sort-method=[manual|alpha|natural|numeric]\n\n";
	echo "Node options:\n";
	echo "    --tree-id=[ID]\n [--parent-node=[ID]]\n\n";
	echo "Header node options:\n";
	echo "    --name=[Name]\n\n";
	echo "Host node options:\n";
	echo "    --host-id=[ID]\n";
	echo "    [--host-group-style=[1|2]]\n";
	echo "    (host group styles:\n";
	echo "     1 = Graph Template,\n";
	echo "     2 = Data Query Index)\n\n";
	echo "Graph node options:\n";
	echo "    --graph-id=[ID]\n";
	echo "    [--rra-id=[ID]]\n\n";
	echo "List Options:\n";
	echo "    --add-tree [Tree options]\n";
	echo "    --add-node-header [Node options] [Header node options]\n";
	echo "    --add-node-host [Node options] [Host node options]\n";
	echo "    --add-node-graph [Node options] [Graph node options]\n";
	echo "    --list-hosts\n";
	echo "    --list-trees\n";
	echo "    --list-nodes --tree-id=[ID]\n";
	echo "    --list-rras\n";
	echo "    --list-graphs --host-id=[ID]\n";
}
#####################################################################
#
# Main
#
/* process calling arguments */
$parms = $_SERVER["argv"];
array_shift($parms);

if (!sizeof($parms)){
	display_help();
	exit(1);
}else{
	/* setup defaults */
	$username = "";
	$password = "";
	$switch = "";

	$type       = '';  # tree or node
	$name       = '';  # Name of a tree or node
	$sortMethod = 'a'; # a = alpha, n = numeric, m = manual
	$parentNode = 0;   # When creating a node, the parent node of this node (or zero for root-node)
	$treeId     = 0;   # When creating a node, it has to go in a tree
	$nodeType   = '';  #  Should be 'header', 'graph' or 'host' when creating a node
	$graphId    = 0;   # The ID of the graph to add (gets added to parentNode)
	$rra_id     = 0;   # The rra_id for the graph to display: 1 = daily, 2 = weekly, 3 = monthly, 4 = yearly

	$hostId         = 0;
	$hostGroupStyle = 1; # 1 = Graph Template,  2 = Data Query Index

	foreach($parms as $parameter) {
		@list($arg, $value) = @explode("=", $parameter);

		switch($arg)
		{
			case "--username":
				$username = trim($value);
				break;
			case "--password":
				$password = trim($value);
				break;
			case "--urn":
				$urn = trim($value);
				break;
			case "--name":
				$name = trim($value);
				break;
			case "--sort-method":
				$sortMethod = trim($value);
				break;
			case "--parent-node":
				$parentNode = trim($value);
				break;
			case "--tree-id":
				$treeId = trim($value);
				break;
			case "--node-type":
				$nodeType = trim($value);
				break;
			case "--graph-id":
				$graphId = trim($value);
				break;
			case "--rra-id":
				$rra_id = trim($value);
				break;
			case "--host-id":
				$hostId = trim($value);
				break;
			case "--host-group-style":
				$hostGroupStyle = trim($value);
				break;
			case "--do":
				$switch = trim($value);
				break;
			default:
				usage();
				exit(0);
		}
	}
	
	if ($username == "" || $password == ""){
		display_help();
		exit(1);
	}
	if ($urn == ""){
		display_help();
		exit(1);
	}
	$url_Location = 'http://' . $urn . '/soap_service.php';
	$urn_Location = 'urn:' . $urn . '';

	$opts = array('location' => $url_Location,
	'uri' => 'pc_SOAP_return_soap');
	$client = new SOAPClient(null, $opts);

	$auth = new SOAPAuth ($username, $password);
	$header = new SOAPHeader($urn_Location, 'authenticate_user', $auth);
	
try{
	switch($switch)
	{
		case "--list-hosts":
			$returnResult = $client->__soapCall('return_list_IDHostname_get_Hosts', array(), array(), array($header));
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){display_help(); printThrowException($returnResult["Errors"]);exit(1);}
			displayHosts($returnResult["Result"]);
			exit(0);
			break;
		case "--list-trees":
			$returnResult = $client->__soapCall('return_list_IDName_get_Trees', array(), array(), array($header));
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){display_help(); printThrowException($returnResult["Errors"]);exit(1);}
			displayTrees($returnResult["Result"]);
			exit(0);
			break;
		case "--list-nodes":
			$returnResult = $client->__soapCall('return_list_IDTitle_get_TreeNodes', array('treeId' => $treeId), array(), array($header));
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){display_help(); printThrowException($returnResult["Errors"]);exit(1);}
			displayTreeNodes($returnResult["Result"], $treeId);
			exit(0);
			break;
		/*case "--list-rras":
			$returnResult = $clientTree->__soapCall('return_list_get_rras',
			array(), array(), array($header));
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){display_help(); printThrowException($returnResult["Errors"]);exit(1);}
			displayRRAs($returnResult["Result"]);
			exit(0);
			break;*/
		/*case "--list-graphs":
			$returnResult = $clientTree->__soapCall('return_list_get_graphs',
			array('hostId' => $hostId), array(), array($header));
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){display_help(); printThrowException($returnResult["Errors"]);exit(1);}
			displayHostGraphs($hostId, $returnResult["Result"]);
			exit(0);
			break;*/
		case "--add-tree":
			$returnResult = $clientTree->__soapCall('return_ID_add_tree',
			array('name' => $name, 'sortMethod' => $sortMethod), array(), array($header));
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){display_help(); printThrowException($returnResult["Errors"]);exit(1);}
			printf("Added tree tree-id: (%d)\n", $returnResult["Result"]);
			exit(0);
			break;
		case "--add-node-header":
			$returnResult = $clientTree->__soapCall('return_ID_add_node_header',
			array('name' => $node_name, 'parentNode' => $parentNode, 'treeId' => $treeId), array(), array($header));
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){display_help(); printThrowException($returnResult["Errors"]);exit(1);}
			printf("Added Node Header node-id: (%d)\n", $returnResult["Result"]);
			exit(0);
			break;
		case "--add-node-host":
			$returnResult = $clientTree->__soapCall('return_ID_add_node_host',
			array('name' => $name, 'parentNode' => $parentNode, 'treeId' => $treeId,
			'hostId' => $hostId, 'hostGroupStyle' => $hostGroupStyle), array(), array($header));
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){display_help(); printThrowException($returnResult["Errors"]);exit(1);}
			printf("Added Node Host node-id: (%d)\n", $returnResult["Result"]);
			exit(0);
			break;
		case "--add-node-graph":
			$returnResult = $clientTree->__soapCall('return_ID_add_node_graph',
			array('parentNode' => $parentNode, 'treeId' => $treeId, 'graphId' => $graphId, 'rra_id' => $rra_id), array(), array($header));
			if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){display_help(); printThrowException($returnResult["Errors"]);exit(1);}
			printf("Added Node Graph node-id: (%d)\n", $returnResult["Result"]);
			exit(0);
			break;
		default:
			display_help();
			break;
	}
	exit(0);
	
} catch (Exception $e) {
	echo ("Message = " . $e->__toString() . "\n");	
	exit(1);
}

}

function displayTrees($trees)
{
	echo "Known Trees:(id, name)\n";
	foreach ($trees as $elemnt) {
		echo $elemnt["id"] . "\t" . $elemnt["name"] ."\n";
	}
}

function displayTreeNodes($treeNodes,$treeId)
{
	echo "Known Tree Nodes for Tree ID $treeId:(id, title)\n";
	foreach ($treeNodes as $elemnt) {
		echo $elemnt["id"] . "\t" . $elemnt["title"] ."\n";
	}
}

function displayHosts($hosts)
{
	echo "Known Hosts:(id, hostname)\n";
	foreach ($hosts as $id => $name) {
		echo "$id\t$name\n";
	}
}

class SOAPAuth{
	public $username;
	public $password;

	public function __construct($username, $password){
		$this->username = $username;
		$this->password = $password;
	}
}

function printThrowException($errors){
	foreach($errors as $throwException){
		printf ("Error: " . $throwException["description"] . " - " . $throwException["function"] . "/" . $throwException["variable"] . "\n");
	}
}

?>